package com.cardinfolink.smart.pos.sdk.sample;

import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.os.Parcelable;
import android.support.v7.app.AppCompatActivity;
import android.text.TextUtils;
import android.text.method.ScrollingMovementMethod;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.EditText;
import android.widget.PopupWindow;
import android.widget.TextView;
import android.widget.Toast;

import com.cardinfolink.pos.bean.PrinterResult;
import com.cardinfolink.pos.listener.Callback;
import com.cardinfolink.pos.listener.ProgressCallback;
import com.cardinfolink.pos.sdk.CILRequest;
import com.cardinfolink.pos.sdk.CILResponse;
import com.cardinfolink.pos.sdk.CILSDK;
import com.cardinfolink.pos.sdk.constant.TransConstants;
import com.cardinfolink.pos.sdk.model.Trans;
import com.cardinfolink.sdk.common.CFCardSDK;
import com.cardinfolink.sdk.constant.Constants;
import com.cardinfolink.smart.pos.sdk.utils.SettleDaoUtil;
import com.cardinfolink.smart.pos.sdk.utils.ToastUtils;
import com.google.gson.Gson;

import java.util.Locale;

public class MainActivity extends AppCompatActivity {

    TextView tvMsg, etMerCode, etTermCode;
    PopupWindow popupWindow;
    String merCode, termCode;
    private CILResponse qrResult;//扫码消费结果


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        CILSDK.connect(this);
        tvMsg = (TextView) findViewById(R.id.tv_msg);
        tvMsg.setMovementMethod(ScrollingMovementMethod.getInstance());
        etMerCode = (TextView) findViewById(R.id.tv_mer_code);
        etTermCode = (TextView) findViewById(R.id.tv_term_code);

        CILResponse.Info systemParams = CILSDK.getSystemParams();
        if (systemParams != null){
            String merCode = systemParams.getMerCode();
            String termCode = systemParams.getTermCode();
            if (!TextUtils.isEmpty(merCode)){
                etMerCode.setText(merCode);
                etMerCode.setEnabled(false);
            }
            if (!TextUtils.isEmpty(termCode)){
                etTermCode.setText(termCode);
                etTermCode.setEnabled(false);
            }

        }
    }

    public void activate(View view) {
        merCode = etMerCode.getText().toString();
        termCode = etTermCode.getText().toString();
        if (TextUtils.isEmpty(merCode) || TextUtils.isEmpty(termCode)) {
            Toast.makeText(getApplicationContext(), "参数不能为空", Toast.LENGTH_SHORT).show();
            return;
        }
        CILSDK.active(merCode, termCode, new Callback<CILResponse>() {
            @Override
            public void onResult(CILResponse cilResponse) {
                if (cilResponse != null && cilResponse.getStatus() == 0) {
                    appendMsg("激活成功");
                } else {
                    appendMsg(cilResponse == null ? "激活失败" : cilResponse.getMessage());
                }
            }

            @Override
            public void onError(Parcelable cilRequest, Exception e) {
                appendMsg(e.getMessage());
            }
        });
    }

    public void downloadParams(View view) {
        CILResponse.Info systemParams = CILSDK.getSystemParams();
        if (systemParams != null) {
            merCode = systemParams.getMerCode();
            termCode = systemParams.getTermCode();
        }
        if (TextUtils.isEmpty(merCode)) {
            merCode = etMerCode.getText().toString();
        }
        if (TextUtils.isEmpty(termCode)) {
            termCode = etTermCode.getText().toString();
        }
        if (TextUtils.isEmpty(merCode) || TextUtils.isEmpty(termCode)) {
            Toast.makeText(getApplicationContext(), "参数不能为空", Toast.LENGTH_SHORT).show();
            return;
        }
        CILSDK.downloadParams(merCode, termCode, new Callback<CILResponse>() {
            @Override
            public void onResult(CILResponse cilResponse) {
                if (cilResponse != null && cilResponse.getStatus() == 0) {
                    appendMsg("参数下载成功");
                } else {
                    appendMsg(cilResponse == null ? "参数下载失败" : cilResponse.getMessage());
                }
            }

            @Override
            public void onError(Parcelable cilRequest, Exception e) {
                appendMsg(e.getMessage());
            }
        });
    }

    public void init(View view) {
        final ProgressDialog dialog = new ProgressDialog(this);
        dialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
        dialog.setTitle("初始化");
        dialog.setMax(100);
        dialog.setProgress(0);
        dialog.setButton(DialogInterface.BUTTON_POSITIVE, "确认", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                dialog.dismiss();
            }
        });
        dialog.show();
        CILSDK.downloadParamsWithProgress(new ProgressCallback<CILResponse>() {
            @Override
            public void onProgressUpdate(int progress) {
                dialog.setProgress(progress);
            }

            @Override
            public void onResult(CILResponse cilResponse) {
                dialog.setTitle("初始化完成");
            }

            @Override
            public void onError(Parcelable cilRequest, Exception e) {
                dialog.setTitle(e.getMessage());
            }
        });

    }


    public void consume(View view) {
        View inflate = getLayoutInflater().inflate(R.layout.consume_pop, null, false);
        final EditText etAmount = (EditText) inflate.findViewById(R.id.et_amount);
        inflate.findViewById(R.id.btn_confirm).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                CFCardSDK.consumeOfBank(MainActivity.this, etAmount.getText().toString());
            }
        });
        showPopup(inflate);
    }

    public void cancel(View view) {
        View inflate = getLayoutInflater().inflate(R.layout.consume_pop, null, false);
        final EditText etAmount = (EditText) inflate.findViewById(R.id.et_amount);
        etAmount.setHint("凭证号");
        inflate.findViewById(R.id.btn_confirm).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                final ProgressDialog dialog = new ProgressDialog(MainActivity.this);
                dialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
                dialog.show();
                CILSDK.getBillByTraceNum(etAmount.getText().toString(), new Callback<CILResponse>() {
                    @Override
                    public void onResult(CILResponse cilResponse) {
                        if (cilResponse != null && cilResponse.getTxn() != null && cilResponse.getTxn()[0] != null) {
                            CFCardSDK.revokeOfBank(MainActivity.this, cilResponse.getTxn()[0]);
                        }
                        dialog.dismiss();
                    }

                    @Override
                    public void onError(Parcelable cilRequest, Exception e) {
                        dialog.dismiss();
                    }
                });
            }
        });
        showPopup(inflate);
    }

    public void refund(View view) {
        View inflate = getLayoutInflater().inflate(R.layout.refund_pop, null, false);
        final EditText etAmount = (EditText) inflate.findViewById(R.id.et_amount);
        final EditText etRefNum = (EditText) inflate.findViewById(R.id.et_ref_num);
        final EditText etDate = (EditText) inflate.findViewById(R.id.et_date);
        inflate.findViewById(R.id.btn_confirm).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                CFCardSDK.refundOfBank(MainActivity.this, etRefNum.getText().toString()
                        , etDate.getText().toString(), etAmount.getText().toString());
            }
        });
        showPopup(inflate);
    }

    public void preAuth(View view) {
        View inflate = getLayoutInflater().inflate(R.layout.consume_pop, null, false);
        final EditText etAmount = (EditText) inflate.findViewById(R.id.et_amount);
        inflate.findViewById(R.id.btn_confirm).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                CFCardSDK.preAuth(MainActivity.this, etAmount.getText().toString());
            }
        });
        showPopup(inflate);
    }

    public void preAuthCancel(View view) {
        View inflate = getLayoutInflater().inflate(R.layout.refund_pop, null, false);
        final EditText etAmount = (EditText) inflate.findViewById(R.id.et_amount);
        final EditText etRefNum = (EditText) inflate.findViewById(R.id.et_ref_num);
        etRefNum.setHint("授权码");
        final EditText etDate = (EditText) inflate.findViewById(R.id.et_date);
        inflate.findViewById(R.id.btn_confirm).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                CFCardSDK.preAuthCancel(MainActivity.this, etAmount.getText().toString(),
                        etDate.getText().toString(), etRefNum.getText().toString());
            }
        });
        showPopup(inflate);
    }

    public void preAuthComp(View view) {
        View inflate = getLayoutInflater().inflate(R.layout.refund_pop, null, false);
        final EditText etAmount = (EditText) inflate.findViewById(R.id.et_amount);
        final EditText etRefNum = (EditText) inflate.findViewById(R.id.et_ref_num);
        etRefNum.setHint("授权码");
        final EditText etDate = (EditText) inflate.findViewById(R.id.et_date);
        inflate.findViewById(R.id.btn_confirm).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                CFCardSDK.preAuthComp(MainActivity.this, etAmount.getText().toString(),
                        etDate.getText().toString(), etRefNum.getText().toString());
            }
        });
        showPopup(inflate);
    }

    public void preAuthCompCancel(View view) {
        View inflate = getLayoutInflater().inflate(R.layout.consume_pop, null, false);
        final EditText etAmount = (EditText) inflate.findViewById(R.id.et_amount);
        etAmount.setHint("凭证号");
        inflate.findViewById(R.id.btn_confirm).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                final ProgressDialog dialog = new ProgressDialog(MainActivity.this);
                dialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
                dialog.show();
                CILSDK.getBillByTraceNum(etAmount.getText().toString(), new Callback<CILResponse>() {
                    @Override
                    public void onResult(CILResponse cilResponse) {
                        if (cilResponse != null && cilResponse.getTxn() != null && cilResponse.getTxn()[0] != null) {
                            CFCardSDK.preAuthCompCancel(MainActivity.this, cilResponse.getTxn()[0]);
                        }
                        dialog.dismiss();
                    }

                    @Override
                    public void onError(Parcelable cilRequest, Exception e) {
                        dialog.dismiss();
                    }
                });
            }
        });
        showPopup(inflate);
    }

    public void balance(View view) {
        CFCardSDK.balanceOfBank(MainActivity.this);
    }

    public void dcc2edc(View view) {
        View inflate = getLayoutInflater().inflate(R.layout.consume_pop, null, false);
        final EditText etAmount = (EditText) inflate.findViewById(R.id.et_amount);
        etAmount.setHint("凭证号");
        inflate.findViewById(R.id.btn_confirm).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                final ProgressDialog dialog = new ProgressDialog(MainActivity.this);
                dialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
                dialog.show();
                CILSDK.getBillByTraceNum(etAmount.getText().toString(), new Callback<CILResponse>() {
                    @Override
                    public void onResult(CILResponse cilResponse) {
                        if (cilResponse != null && cilResponse.getTxn() != null && cilResponse.getTxn()[0] != null) {
                            CFCardSDK.dcc2edc(MainActivity.this, cilResponse.getTxn()[0]);
                        }
                        dialog.dismiss();
                    }

                    @Override
                    public void onError(Parcelable cilRequest, Exception e) {
                        dialog.dismiss();
                    }
                });
            }
        });
        showPopup(inflate);

    }

    public void consumeQr(View view) {
        View inflate = getLayoutInflater().inflate(R.layout.consume_pop, null, false);
        final EditText etAmount = (EditText) inflate.findViewById(R.id.et_amount);
        etAmount.setHint("输入条码");
        inflate.findViewById(R.id.btn_confirm).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                final ProgressDialog dialog = new ProgressDialog(MainActivity.this);
                dialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
                dialog.show();
                CILRequest request = new CILRequest();
                request.setAmount("0.01");
                request.setScanCodeId(etAmount.getText().toString());
                CILSDK.consumeQr(request, new Callback<CILResponse>() {
                    @Override
                    public void onResult(CILResponse cilResponse) {
                        qrResult = cilResponse;
                        appendMsg("扫码消费结果");
                        appendMsg("response:    " + new Gson().toJson(cilResponse));
                        dialog.dismiss();
                    }

                    @Override
                    public void onError(Parcelable parcelable, Exception e) {
                        e.printStackTrace();
                        dialog.dismiss();
                    }
                });
            }
        });
        showPopup(inflate);
    }

    public void queryQr(View view) {
        if (qrResult == null) {
            ToastUtils.showToast(this, "扫码消费信息为空");
            return;
        }
        final CILResponse response = qrResult;

        String resultCode = qrResult.getTrans().getRespCode();


        if ("09".equals(resultCode) || "98".equals(resultCode)) {
            CILRequest request = new CILRequest();
            request.setBatchNum(response.getTrans().getBatchNum());
            request.setTraceNum(response.getTrans().getTraceNum());
            request.setPeriod(10000L);//10s
            request.setLimitTime(6);//6 次
            request.setReferenceNumber(response.getTrans().getRefNum());
            request.setPosInputStyle(response.getTrans().getPosInputStyle());
            request.setScanCodeId(response.getTrans().getScanCodeId());
            request.setAmount(response.getTrans().getTransAmt());
            CILSDK.queryQr(request, new Callback<CILResponse>() {
                @Override
                public void onResult(CILResponse cilResponse) {
                    appendMsg("扫码查询结果");
                    appendMsg("response:    " + new Gson().toJson(cilResponse));
                }

                @Override
                public void onError(Parcelable cilRequest, Exception e) {
                    e.printStackTrace();
                }
            });
        }
    }


    public void gotoSettles(View view) {
        SettleDaoUtil.getInstance().gotoLiquidation(this);
    }


    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        switch (resultCode) {
            case Constants.RESULT_OK:
                int transType = data.getIntExtra(Constants.TRANS_TYPE, Constants.UNKNOWN);
                Trans trans = data.getParcelableExtra(Constants.CONSUME_RESULT);
                if (Constants.BANK_BALANCE == transType) {
                    appendMsg(trans == null ? "查询失败" : "余额：" + trans.getTransAmt());
                    return;
                }
                appendMsg("交易完成：" + trans.toString());
                print(trans, transType);
                break;
            case Constants.RESULT_ERROR:
                transType = data.getIntExtra(Constants.TRANS_TYPE, Constants.UNKNOWN);
                trans = data.getParcelableExtra(Constants.CONSUME_RESULT);
                appendMsg("交易失败：" + trans.toString());
                break;
            default:
                break;
        }
    }


    private PopupWindow showPopup(View view) {
        popupWindow = new PopupWindow(view, ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT, true);
        popupWindow.setBackgroundDrawable(new ColorDrawable(Color.WHITE));
        popupWindow.showAtLocation(tvMsg, Gravity.CENTER, 0, 0);
        WindowManager.LayoutParams lp = getWindow().getAttributes();
        lp.alpha = 0.7f;
        getWindow().setAttributes(lp);
        popupWindow.setOnDismissListener(new PopupWindow.OnDismissListener() {
            @Override
            public void onDismiss() {
                WindowManager.LayoutParams lp = getWindow().getAttributes();
                lp.alpha = 1.0f;
                getWindow().setAttributes(lp);
            }
        });
        return popupWindow;
    }


    private void print(Trans trans, int transType) {
        CILSDK.printKindsReceipts(trans, 3, getTransCode(transType), CILSDK.RECEIPT_MERCHANT, isForeign(trans.getBillingCurr()), null, new Callback<PrinterResult>() {
            @Override
            public void onResult(PrinterResult printerResult) {

            }

            @Override
            public void onError(Parcelable cilRequest, Exception e) {

            }
        });
    }


    private boolean isForeign(String billingCurr) {
        if (!TextUtils.isEmpty(billingCurr) && !"CNY".equals(billingCurr)) {
            return true;
        }
        return false;
    }

    @CILSDK.FormatTransCode
    private String getTransCode(int transType) {
        @CILSDK.FormatTransCode String transCode = TransConstants.TC_CONSUME;
        switch (transType) {
            //消费
            case Constants.BANK_CONSUME:
                transCode = TransConstants.TC_CONSUME;
                break;
            //撤销
            case Constants.BANK_CANCEL:
                transCode = TransConstants.TC_CANCEL;
                break;
            //退货
            case Constants.BANK_REFUND:
                transCode = TransConstants.TC_REFUND;
                break;
            //余额查询
            case Constants.BANK_BALANCE:
                break;
            //预授权
            case Constants.BANK_AUTH:
                transCode = TransConstants.TC_PRE_AUTH;
                break;
            //预授权撤销
            case Constants.BANK_AUTH_CANCEL:
                transCode = TransConstants.TC_PRE_AUTH_CANCEL;
                break;
            //预授权完成
            case Constants.BANK_AUTH_COMP:
                transCode = TransConstants.TC_PRE_AUTH_COMPLETION;
                break;
            //预授权完成撤销
            case Constants.BANK_AUTH_COMP_CANCEL:
                transCode = TransConstants.TC_PRE_AUTH_COMPLETION_CANCEL;
                break;
            case Constants.DCC_TO_EDC:
                transCode = TransConstants.TC_CONSUME;
                break;
            default:
                transCode = TransConstants.TC_CONSUME;
                break;
        }
        return transCode;
    }

    private void appendMsg(final String msg) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                String s = tvMsg.getText().toString();
                tvMsg.setText(String.format(Locale.getDefault(), "%S%n%S", s, msg));
            }
        });
    }
}
